const express = require('express');

const enrollmentApp = express.Router();

const libs = `${process.cwd()}/libs`;

const log = require(`${libs}/log`)(module);

const squel = require('squel');

const query = require(`${libs}/middlewares/query`);

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

let rqf = new ReqQueryFields();

// Complete range of the enrollments dataset.
// Returns a tuple of start and ending years of the complete enrollments dataset.
enrollmentApp.get('/year_range', (req, res, next) => {
    req.sql.from('matricula')
    .field('MIN(matricula.ano_censo)', 'start_year')
    .field('MAX(matricula.ano_censo)', 'end_year');
    next();
}, query, response('range'));

enrollmentApp.get('/location', (req, res, next) => {
    req.sql = squel.select()
    .field('id')
    .field('descricao', 'name')
    .from('localizacao');
    next();
}, query, response('location'));

// Returns all educational levels avaible
enrollmentApp.get('/education_level', (req, res, next) => {
    req.sql.from('serie_ano')
    .field('id')
    .field('nome', 'name');
    next();
}, query, response('education_level'));

// Returns all adm dependencies
enrollmentApp.get('/adm_dependency', (req, res, next) => {
    req.sql.from('dependencia_adm')
    .field('id')
    .field('nome', 'name')
    .where('id <= 4');
    next();
}, query, response('adm_dependency'));

enrollmentApp.get('/adm_dependency_detailed', (req, res, next) => {
    req.sql.from('dependencia_adm')
    .field('id', 'id')
    .field('nome', 'name');
    next();
}, query, response('adm_dependency_detailed'));

// Return genders
enrollmentApp.get('/gender', (req, res, next) => {
    req.result = [
        {id: 1, name: 'Masculino'},
        {id: 2, name: 'Feminino'}
    ];
    next();
}, response('gender'));

// Return ethnic group
enrollmentApp.get('/ethnic_group', (req, res, next) => {
    req.sql.from('cor_raca')
    .field('id')
    .field('nome', 'name');
    next();
}, query, response('ethnic_group'));

enrollmentApp.get('/period', (req, res, next) => {
    req.sql.from('turma_turno')
    .field('id')
    .field('nome', 'name');
    next();
}, query, response('period'));

rqf.addField({
    name: 'filter',
    field: false,
    where: true
}).addField({
    name: 'dims',
    field: true,
    where: false
}).addValue({
    name: 'adm_dependency',
    table: 'dependencia_adm',
    tableField: 'nome',
    resultField: 'adm_dependency_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'dependencia_adm_id',
        foreignTable: 'matricula'
    }
}).addValue({
    name: 'adm_dependency_detailed',
    table: 'dependencia_adm',
    tableField: 'nome',
    resultField: 'adm_dependency_detailed_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'dependencia_adm_priv',
        foreignTable: 'matricula'
    }
}).addValue({
    name: 'education_level',
    table: 'serie_ano',
    tableField: 'nome',
    resultField: 'education_level_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'serie_ano_id',
        foreignTable: 'matricula'
    }
}).addValue({
    name: 'region',
    table: 'regiao',
    tableField: 'nome',
    resultField: 'region_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'regiao_id',
        foreignTable: 'matricula'
    }
}).addValue({
    name: 'state',
    table: 'estado',
    tableField: 'nome',
    resultField: 'state_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'escola_estado_id',
        foreignTable: 'matricula'
    }
}).addValue({
    name: 'city',
    table: 'municipio',
    tableField: 'nome',
    resultField: 'city_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'escola_municipio_id',
        foreignTable: 'matricula'
    }
}).addValue({
    name: 'school',
    table: 'escola',
    tableField: 'nome_escola',
    resultField: 'school_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: ['id', 'ano_censo'],
        foreign: ['escola_id', 'ano_censo'],
        foreignTable: 'matricula'
    }
}).addValue({
    name: 'location',
    table: 'localizacao',
    tableField: 'descricao',
    resultField: 'location_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'localizacao_id',
        foreignTable: 'matricula'
    }
}).addValue({
    name: 'min_year',
    table: 'matricula',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '>=',
        type: 'integer',
        field: 'ano_censo'
    }
}).addValue({
    name: 'max_year',
    table: 'matricula',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '<=',
        type: 'integer',
        field: 'ano_censo'
    }
}).addValue({
    name: 'gender',
    table: 'matricula',
    tableField: 'sexo',
    resultField: 'gender_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'sexo'
    }
}).addValue({
    name: 'ethnic_group',
    table: 'cor_raca',
    tableField: 'nome',
    resultField: 'ethnic_group_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'cor_raca_id',
        foreignTable: 'matricula'
    }
}).addValue({
    name: 'period',
    table: 'turma',
    tableField: 'turno',
    resultField: 'period_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'turno'
    },
    join: {
        primary: 'id',
        foreign: 'turma_id',
        foreignTable: 'matricula'
    }
});

enrollmentApp.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
    log.debug(req.sql.toParam());
    req.sql.field('COUNT(matricula.id)', 'total')
    .field("'Brasil'", 'name')
    .field('matricula.ano_censo', 'year')
    .from('matricula')
    .group('matricula.ano_censo')
    .order('matricula.ano_censo')
    .where('matricula.tipo=0 OR matricula.tipo=1 OR matricula.tipo=2 OR matricula.tipo=3');
    next();
}, query, (req, res, next) => {
    // ids to strings
    req.result.forEach((result) => {
        if(typeof result.gender_id !== 'undefined') {
            switch (result.gender_id) {
                case 1:
                result.gender_name = 'Masculino';
                break;
                case 2:
                result.gender_name = 'Feminino';
                break;
            }
            delete result.gender_id;
        }
        if(typeof result.period_id !== 'undefined') {
            switch (result.period_id) {
                case 1:
                result.period_name = 'Diurno';
                break;
                case 2:
                result.period_name = 'Noturno';
                break;
                case 3:
                result.period_name = 'Integral';
                break;
                default:
                result.period_name = 'Indefinido';
                break;
            }
            delete result.period_id;
        }
    });
    next();
}, response('enrollment'));

function schoolYearIdToStr(schoolYearId) {
    let schoolYearStr;
    switch(schoolYearId) {
        case 11:
        schoolYearStr = 'Creche - Menor de 1 ano';
        break;
        case 12:
        schoolYearStr = 'Creche - 1 ano';
        break;
        case 13:
        schoolYearStr = 'Creche - 2 anos';
        break;
        case 14:
        schoolYearStr = 'Creche - 3 anos';
        break;
        case 21:
        schoolYearStr = 'Pré-escola - 4 anos';
        break;
        case 22:
        schoolYearStr = 'Pré-escola - 5 anos';
        break;
        case 31:
        schoolYearStr = 'Ens. Fundamental - 1º Ano';
        break;
        case 32:
        schoolYearStr = 'Ens. Fundamental - 1ª série/2º ano';
        break;
        case 33:
        schoolYearStr = 'Ens. Fundamental - 2ª série/3º ano';
        break;
        case 34:
        schoolYearStr = 'Ens. Fundamental - 3ª série/4º ano';
        break;
        case 35:
        schoolYearStr = 'Ens. Fundamental - 4ª série/5º Ano';
        break;
        case 41:
        schoolYearStr = 'Ens. Fundamental - 5ª série/6º ano';
        break;
        case 42:
        schoolYearStr = 'Ens. Fundamental - 6ª série/7º ano';
        break;
        case 43:
        schoolYearStr = 'Ens. Fundamental - 7ª série/8º ano';
        break;
        case 44:
        schoolYearStr = 'Ens. Fundamental - 8ª serie/9º ano';
        break;
        case 51:
        schoolYearStr = 'Ens. Médio - 1ª série'; // equivalent to 'EM 1 Série'
        break;
        case 52:
        schoolYearStr = 'Ens. Médio - 2ª série'; // equivalent to 'EM 2 Série'
        break;
        case 53:
        schoolYearStr = 'Ens. Médio - 3ª série'; // equivalent to 'EM 3 Série'
        break;
        case 54:
        schoolYearStr = 'Ens. Médio - 4ª série'; // equivalent to 'EM 4 Série'
        break;
        case 61:
        schoolYearStr = 'EJA - anos iniciais do Ens. Fundamental';
        break;
        case 62:
        schoolYearStr = 'EJA - anos finais do Ens. Fundamental';
        break;
        case 63:
        schoolYearStr = 'EJA - Ensino Médio';
        break;
        case 64:
        schoolYearStr = 'EJA semi-presencial';
        break;
        case 71:
        schoolYearStr = 'Educação Profissional';
        break;
        default:
        schoolYearStr = 'Não classificado';
    }
    return schoolYearStr;
}

let simRqf = new ReqQueryFields();

simRqf.addField({
    name: 'filter'
}).addValue({
    name: 'simulation_time'
})

enrollmentApp.get('/offer_projection', simRqf.parse(), (req, res, next) => {
    const max_year = squel.select()
    .field('MAX(matricula.ano_censo)')
    .from('matricula')
    .toString();

    req.sql.field('matricula.ano_censo', 'offerYear')
    .field('turma.turno', 'period_id')
    .field('etapa_ensino.desc_etapa', 'education_level_name')
    .field('etapa_ensino.id', 'education_level_id')
    .field('matricula.serie_ano_id')
    .field('COUNT(matricula.id)', 'currentOffer')
    .from('matricula')
    .join('etapa_ensino', null, 'matricula.etapa_ensino_id=etapa_ensino.id')
    .join('turma', null, 'matricula.turma_id=turma.id')
    .where('matricula.tipo <= 3')
    .where(`matricula.ano_censo IN (${max_year})`)
    .where('turma.turno <= 2')
    .group('matricula.ano_censo')
    .group('turma.turno')
    .group('etapa_ensino.desc_etapa')
    .group('etapa_ensino.id')
    .group('matricula.serie_ano_id');

    next();
}, query, (req, res, next) => {
    let years = parseInt(req.filter.simulation_time, 10);
    if(isNaN(years)) years = 0;
    let offerYear = req.result[0].offerYear;
    let temp = {daytime: {}, nightly: {}};
    let result = {
        daytime: [],
        nightly: [],
        offerYear,
        years: Array.apply(null, {length: years}).map(Number.call, Number).map((i)=>i+offerYear+1)
    };
    req.result.forEach((i) => {
        let period = (i.period_id === 1) ? 'daytime' : 'nightly';
        if(typeof temp[period][i.education_level_name] === 'undefined') {
            temp[period][i.education_level_name] = {
                id: i.education_level_id,
                name: i.education_level_name,
                currentOffer: 0,
                grades: []
            };
        }
        temp[period][i.education_level_name].currentOffer += i.currentOffer;
        temp[period][i.education_level_name].grades.push({
            id: i.serie_ano_id,
            name: schoolYearIdToStr(i.serie_ano_id),
            currentOffer: i.currentOffer
        });
    });
    Object.keys(temp.daytime).forEach((k) => {
        result.daytime.push(temp.daytime[k]);
    });
    Object.keys(temp.nightly).forEach((k) => {
        result.nightly.push(temp.nightly[k]);
    });
    req.result = result;
    next();
}, response('offer_projection'));

module.exports = enrollmentApp;
